from datetime import datetime
import sqlite3
from typing import Any, Dict, List, Tuple

from ..config import dexie_db_path
from ..decorators.with_db_connection import get_full_db_path, with_db_connection

CREATE_OFFERS = """
    CREATE TABLE IF NOT EXISTS offers (
        trade_id TEXT PRIMARY KEY,
        offer_id TEXT NOT NULL,
        status INTEGER NOT NULL,
        has_rewards INTEGER NOT NULL DEFAULT 0,
        ts TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
        ts_checked TIMESTAMP,
        ts_claimed TIMESTAMP,
        ts_inactive TIMESTAMP
    );
"""


async def create_db(fingerprint: int) -> None:
    db = sqlite3.connect(get_full_db_path(dexie_db_path, fingerprint))
    db.execute(CREATE_OFFERS)
    db.close()


@with_db_connection(dexie_db_path)  # type: ignore
async def insert_new_offers(conn: Any, offers: List[Tuple[str, str, int]]) -> int:
    # insert offers if needed
    async with conn.cursor() as cursor:
        sql = "INSERT OR IGNORE INTO offers (trade_id, offer_id, status, ts_inactive) VALUES (?, ?, ?, ?)"
        values = [
            (trade_id, offer_id, status, None if status == 0 else datetime.now())
            for trade_id, offer_id, status in offers
        ]
        result = await cursor.executemany(sql, values)

    await conn.commit()
    return result.rowcount


@with_db_connection(dexie_db_path)  # type: ignore
async def update_offers_status(conn: Any, offers: List[Tuple[str, str, int]]) -> int:
    counts = 0
    for trade_id, _, status in offers:
        if status == 0:
            continue

        # update status and ts_inactive
        # for offers that are no longer active (completed or cancelled)
        # and ts_inactive is null
        ts_inactive = datetime.now()
        async with conn.cursor() as cursor:
            sql = """
            UPDATE offers 
            SET status = ?, ts_inactive = ? 
            WHERE trade_id = ? 
            AND ts_inactive IS NULL"""
            result = await cursor.execute(sql, (status, ts_inactive, trade_id))
            counts += result.rowcount

    await conn.commit()
    return counts


@with_db_connection(dexie_db_path)  # type: ignore
async def update_offers_rewards(conn: Any, offers: List[Dict[str, Any]]) -> int:
    counts = 0
    for offer in offers:
        status = offer["status"]

        # update status, has_rewards, and ts_claimed
        async with conn.cursor() as cursor:
            sql = """
            UPDATE offers 
            SET status = ?, 
                ts_claimed = NULL, 
                has_rewards = 1 
            WHERE offer_id = ?"""
            result = await cursor.execute(sql, (status, offer["id"]))
            counts += result.rowcount

    await conn.commit()
    return counts


@with_db_connection(dexie_db_path)  # type: ignore
async def update_offers_ts_checked(
    conn: Any, offers: List[Tuple[str, str, int]]
) -> int:
    for trade_id, _, _ in offers:
        ts_checked = datetime.now()
        async with conn.cursor() as cursor:
            sql = """
            UPDATE offers 
            SET ts_checked = ?
            WHERE trade_id = ?"""
            result = await cursor.execute(sql, (ts_checked, trade_id))

    await conn.commit()


@with_db_connection(dexie_db_path)  # type: ignore
async def update_offers_ts_claimed(conn: Any, offers: List[str]) -> int:
    for offer_id in offers:
        ts_claimed = datetime.now()
        async with conn.cursor() as cursor:
            sql = """
            UPDATE offers 
            SET ts_claimed = ?, has_rewards = 0
            WHERE offer_id = ?"""
            result = await cursor.execute(sql, (ts_claimed, offer_id))

    await conn.commit()


@with_db_connection(dexie_db_path)  # type: ignore
async def get_claimable_offers(conn: Any) -> List[str]:
    offer_ids: List[str] = []
    async with conn.execute(
        """
        SELECT offer_id 
        FROM offers 
        WHERE ts_checked IS NULL            -- unchecked offers
        OR ts_inactive IS NULL              -- active offers
        OR (ts_inactive IS NOT NULL 
            AND has_rewards = 1)            -- inactive offers with unclaimed rewards
        OR (ts_inactive IS NOT NULL 
            AND ts_checked < ts_inactive)   -- unchecked inactive offers
        LIMIT 1000
    """
    ) as cursor:
        rows = await cursor.fetchall()
        for row in rows:
            offer_ids.append(row[0])
    return offer_ids
